import json

import pika


class RabbitMQClient(object):

    def __init__(self, host, port, user, password, vhost, queue_name):
        credentials = pika.PlainCredentials(user, password)
        parameters = pika.ConnectionParameters(
            host=host, port=port,
            virtual_host=vhost, credentials=credentials
        )
        self.queue_name = queue_name
        self.connection = pika.BlockingConnection(parameters)
        self.channel = self.connection.channel()

    def read_messages(self, number_messages=1):
        messages = []
        while True:
            try:
                message = self.get_message()
            except StopIteration:
                if messages:
                    yield messages
                break
            else:
                messages.append(message)
                if len(messages) == number_messages:
                    yield messages

    def get_message(self):
        message = self._consumer().next()
        if isinstance(message, dict):
            return message

    def _consumer(self):
        while True:
            method_frame, _, body = self.channel.basic_get(self.queue_name)
            if method_frame:
                self.channel.basic_ack(method_frame.delivery_tag)
                body = json.loads(body)
                yield body
            else:
                break

    def close(self):
        if not self.channel.is_closed:
            self.channel.close()
        if not self.connection.is_closed:
            self.connection.close()



import os

ACS_API_URL = os.getenv('ACS_API_URL')
ACS_API_KEY = os.getenv('ACS_API_KEY')
ACS_API_SECRET_KEY = os.getenv('ACS_API_SECRET_KEY')
ACS_RMQ_USER = os.getenv('ACS_RMQ_USER')
ACS_RMQ_PASSWORD = os.getenv('ACS_RMQ_PASSWORD')
ACS_RMQ_HOST = os.getenv('ACS_RMQ_HOST')
ACS_RMQ_PORT = int(os.getenv('ACS_RMQ_PORT', 5672))
ACS_RMQ_QUEUE = os.getenv('ACS_RMQ_QUEUE')
ACS_RMQ_VIRTUAL_HOST = os.getenv('ACS_RMQ_VIRTUAL_HOST')



from setuptools import setup

VERSION = __import__('globomap_driver_acs').VERSION

setup(
    name='globomap-driver-acs',
    version=VERSION,
    description='Python library for globomap-driver to get data '
                'from Cloudstack',
    author='Victor Mendes Eduardo',
    author_email='victor.eduard@corp.globo.com',
    install_requires=[
        'pika==0.10.0',
        'python-dateutil==2.4.2',
    ],
    url='https://github.com/globocom/globomap-driver-acs',
    packages=['globomap_driver_acs'],
)



import logging
import time
from dateutil.parser import parse
from cloudstack import CloudStackClient, CloudstackService
from rabbitmq import RabbitMQClient
from settings import ACS_RMQ_USER, ACS_RMQ_PASSWORD, \
    ACS_RMQ_HOST, ACS_RMQ_PORT, ACS_RMQ_QUEUE, \
    ACS_RMQ_VIRTUAL_HOST, ACS_API_URL, ACS_API_KEY, ACS_API_SECRET_KEY


class Cloudstack(object):

    log = logging.getLogger(__name__)

    def __init__(self):
        self.rabbitmq = RabbitMQClient(
            host=ACS_RMQ_HOST,
            port=ACS_RMQ_PORT,
            user=ACS_RMQ_USER,
            password=ACS_RMQ_PASSWORD,
            vhost=ACS_RMQ_VIRTUAL_HOST,
            queue_name=ACS_RMQ_QUEUE
        )

    def updates(self, number_messages=1):
        messages = []
        while True:
            update = None
            try:
                message = self.rabbitmq.get_message()
                if message:
                    update = self._format_update(message)

            except StopIteration:
                if messages:
                    yield messages
                raise StopIteration
            else:
                if update:
                    messages.append(update)
                if len(messages) == number_messages:
                    yield messages
                    messages = []

                if not update:
                    break

    def _format_update(self, msg):
        create_event = msg["event"] == "VM.CREATE"
        event_completed = msg["status"] == "Completed"

        if event_completed and create_event:
            vm = self._get_virtual_machine_data(msg["entityuuid"], msg)
            if not vm:
                return
            update = {
                "action": "CREATE",
                "collection": "comp_unit",
                "type": "collections",
                "element": vm
            }
            return update

    def _get_virtual_machine_data(self, id, msg):
        cloudstack_service = self._get_cloudstack_service()
        virtual_machine = cloudstack_service.get_virtual_machine(id)
        if virtual_machine:
            project = cloudstack_service.get_project(
                virtual_machine["projectid"]
            )
            account = project['account']
            element = {
                "id": virtual_machine["displayname"],
                "name": virtual_machine["displayname"],
                "timestamp": self._get_event_time(msg["eventDateTime"]),
                "properties": [
                    {
                        "key": "uuid",
                        "value": virtual_machine["id"]
                    },
                    {
                        "key": "zone",
                        "value": virtual_machine["zonename"]
                    },
                    {
                        "key": "service_offering",
                        "value": virtual_machine["serviceofferingname"]
                    },
                    {
                        "key": "template",
                        "value": virtual_machine["templatename"]
                    },
                    {
                        "key": "project",
                        "value": virtual_machine["project"]
                    },
                    {
                        "key": "account",
                        "value": account
                    }
                ]
            }
            return element

    def _get_event_time(self, event_time):
        timetuple = parse(event_time).replace(tzinfo=None).timetuple()
        return int(time.mktime(timetuple))

    def _get_cloudstack_service(self):
        acs_client = CloudStackClient(
            ACS_API_URL, ACS_API_KEY,
            ACS_API_SECRET_KEY, True
        )
        return CloudstackService(acs_client)



# -*- coding: utf-8 -*-
# By: Kelcey Damage, 2012 & Kraig Amador, 2012
import hashlib
import hmac
import base64
import urllib
import json
import sys
import ssl


class SignedAPICall(object):

    def __init__(self, api_url, apiKey, secret, verifysslcert):
        self.api_url = api_url
        self.apiKey = apiKey
        self.secret = secret
        self.verifysslcert = verifysslcert

    def request(self, args, action):
        args['apiKey'] = self.apiKey

        self.params = []
        self._sort_request(args)
        self._create_signature()
        self._build_post_request(action)

    def _sort_request(self, args):
        keys = sorted(args.keys())
        for key in keys:
            self.params.append(key + '=' + urllib.quote_plus(args[key]))

    def _create_signature(self):
        self.query = '&'.join(self.params).replace("+", "%20")\
            .replace(":", "%3A")
        digest = hmac.new(
            self.secret,
            msg=self.query.lower(),
            digestmod=hashlib.sha1).digest()

        self.signature = base64.b64encode(digest)

    def _build_post_request(self, action='GET'):
        self.query += '&signature=' + urllib.quote_plus(self.signature)
        self.value = self.api_url
        if action == 'GET':
            self.value += '?' + self.query


class CloudStackClient(SignedAPICall):

    def __getattr__(self, name):
        def handlerFunction(*args, **kwargs):
            args = list(args)
            if len(args) == 1:
                args.insert(0, 'GET')
            action = args[0] or 'GET'
            if kwargs:
                return self._make_request(name, kwargs)
            return self._make_request(name, args[1], action)
        return handlerFunction

    def _http_get(self, url):
        if self.verifysslcert and sys.version_info < (2, 7, 9):
            response = urllib.urlopen(url)
        else:
            ctx = ssl.create_default_context()
            ctx.check_hostname = False
            ctx.verify_mode = ssl.CERT_NONE

            response = urllib.urlopen(url, context=ctx)
        return response.read()

    def _http_post(self, url, data):
        if self.verifysslcert and sys.version_info < (2, 7, 9):
            response = urllib.urlopen(url, data)
        else:
            ctx = ssl.create_default_context()
            ctx.check_hostname = False
            ctx.verify_mode = ssl.CERT_NONE
            response = urllib.urlopen(url, data, context=ctx)
        return response.read()

    def _make_request(self, command, args, action='GET'):
        args['response'] = 'json'
        args['command'] = command
        self.request(args, action)

        tries = 3
        while True:
            try:
                if action == 'GET':
                    data = self._http_get(self.value)
                else:
                    data = self._http_post(self.value, self.query)
                break
            except IOError, e:
                tries -= 1
                if not tries:
                    raise e

        key = command.lower() + "response"
        if key == 'deletenetworkinglobonetworkresponse':
            key = 'deletenetworkresponse'
        if key == 'listglobonetworkpoolsresponse':
            key = 'listglobonetworkpoolresponse'
        if key == 'acquirenewlbipresponse':
            key = 'associateipaddressresponse'
        if key == 'listcountersresponse':
            key = 'counterresponse'
        if key == 'createconditionresponse':
            key = 'conditionresponse'
        if key == 'createautoscalepolicyresponse':
            key = 'autoscalepolicyresponse'
        if key == 'createautoscalevmprofileresponse':
            key = 'autoscalevmprofileresponse'
        if key == 'createautoscalevmgroupresponse':
            key = 'autoscalevmgroupresponse'
        if key == 'enableautoscalevmgroupresponse':
            key = 'enableautoscalevmGroupresponse'

        return json.loads(data)[key]


class CloudstackService(object):

    def __init__(self, cloudstack_client):
        self.cloudstack_client = cloudstack_client

    def get_virtual_machine(self, id):
        virtual_machines = self.cloudstack_client.\
            listVirtualMachines({'id': id, 'listall': 'true'})
        if virtual_machines['count'] == 1:
            return virtual_machines['virtualmachine'][0]

    def get_project(self, id):
        projects = self.cloudstack_client.\
            listProjects({'id': id, 'listall': 'true'})
        if projects['count'] == 1:
            return projects['project'][0]
